package me.mctp.managers;

import java.security.SecureRandom;
import java.util.UUID;
import java.util.WeakHashMap;

public class PinManager {
    private static WeakHashMap<UUID, String> pins = new WeakHashMap<>();

    /**
     * Get the pin of a player
     *
     * @param pUUID The player UUID
     *
     * @return The pin
     */
    public static String getPIN(UUID pUUID) {
        if (pUUID == null) return null;

        if (!pins.containsKey(pUUID)) {
            StringBuilder builder = new StringBuilder();
            SecureRandom random = new SecureRandom();
            for (int i = 0; i < 9; ++i) {
                builder.append(random.nextInt(9));
            }
            String pin = builder.toString();

            pins.put(pUUID, pin);
            return pin;
        } else {
            return pins.get(pUUID);
        }
    }

    /**
     * Check if a player has a pin
     *
     * @param pUUID The player UUID
     *
     * @return The pin
     */
    public static boolean hasPin(UUID pUUID) {
        return pUUID != null && pins.containsKey(pUUID);
    }

    /**
     * Check if the pin is correct
     *
     * @param pUUID The player UUID
     * @param pin The pin
     *
     * @return true/false
     */
    public static boolean checkPin(UUID pUUID, String pin) {
        if (pUUID == null || pin == null || pin.isEmpty()) {
            return false;
        }

        return pins.containsKey(pUUID) && pin.equals(pins.get(pUUID));
    }

    /**
     * Remove the pin
     *
     * @param pUUID The player UUID
     */
    public static void removePin(UUID pUUID) {
        if (pUUID != null) {
            pins.remove(pUUID);
        }
    }
}
